<?php

namespace app\controller;

use app\BusinessException;
use app\model\UserModel;
use app\service\UserService;
use app\validate\UserValidate;
use think\db\exception\DbException;
use think\facade\Db;
use think\facade\Request;
use think\response\Json;

class User extends Base
{
    public function save(): Json
    {
        $param['account'] = Request::param('account/s', '', 'trim');
        $param['password'] = Request::param('password/s', '');
        $param['name'] = Request::param('name/s', '', 'trim');
        validate(UserValidate::class)->scene('save')->check($param);
        UserModel::create([
            'account' => $param['account'],
            'password' => password_hash($param['password'], PASSWORD_DEFAULT),
            'name' => $param['name'],
        ]);
        return success();
    }

    /**
     * @param $id
     * @return Json
     * @throws DbException
     */
    public function delete($id): Json
    {
        UserModel::find($id)->delete();
        return success();
    }

    public function update($id): Json
    {
        $param['user_id'] = $id;
        $param['account'] = Request::param('account/s', '', 'trim');
        $param['password'] = Request::param('password/s', '');
        $param['name'] = Request::param('name/s', '', 'trim');
        validate(UserValidate::class)->scene('edit')->check($param);

        $oldUser = UserModel::where('user_id', '=', $param['user_id'])->find();
        if (empty($oldUser)) return error([], '运营账户找不到');

        $query[] = ['user_id', '=', $param['user_id']];
        $data = [];
        if ($oldUser['account'] != $param['account']) $data['account'] = $param['account'];
        if (!empty($param['password']) && !password_verify($param['password'], $oldUser['password'])) {
            $data['password'] = password_hash($param['password'], PASSWORD_DEFAULT);
        }
        if ($oldUser['name'] != $param['name']) $data['name'] = $param['name'];
        if (empty($data)) return success([], '没有修改任何内容');
        UserModel::where($query)->save($data);

        return success([], '修改成功');
    }

    /**
     * @param $id
     * @return Json
     * @throws DbException
     */
    public function identity($id): Json
    {
        $param['identity'] = Request::param('identity/d', 0);
        validate(UserValidate::class)->scene('identity')->check($param);

        $oldUser = UserModel::where('user_id', '=', $id)->find();
        if (empty($oldUser)) return error([], '运营账户找不到');

        $query[] = ['user_id', '=', $id];
        $data = [];
        if ($oldUser['identity'] != $param['identity']) $data['identity'] = $param['identity'];
        if (empty($data)) return success([], '没有修改任何内容');
        UserModel::where($query)->save($data);

        return success([], '修改成功');
    }

    /**
     * @param $id
     * @return Json
     * @throws DbException
     */
    public function disableSwitch($id): Json
    {
        $param['disable_switch'] = Request::param('disable_switch/d', 0);
        validate(UserValidate::class)->scene('disable_switch')->check($param);

        $oldUser = UserModel::where('user_id', '=', $id)->find();
        if (empty($oldUser)) return error([], '运营账户找不到');

        $query[] = ['user_id', '=', $id];
        $data = [];
        if ($oldUser['disable_switch'] != $param['disable_switch']) $data['disable_switch'] = $param['disable_switch'];
        if (empty($data)) return success([], '没有修改任何内容');
        UserModel::where($query)->save($data);

        return success([], '修改成功');
    }

    /**
     * @param $id
     * @return Json
     * @throws DbException
     */
    public function read($id): Json
    {
        $user = UserModel::where('user_id', $id)->find();
        unset($user['password']);
        return success($user);
    }

    /**
     * @return Json
     * @throws DbException
     */
    public function index(): Json
    {
        $param['limit'] = Request::param('limit/d', 10);
        $param['name'] = Request::param('name/s');
        $param['account'] = Request::param('account/s');
        $param['keyWords'] = Request::param('keyWords/s');

        $query = [];
        $query[] = ['delete_time', '=', 0];
        if ($param['name'] !== null && trim($param['name']) != '') $query[] = ['name', '=', trim($param['name'])];
        if ($param['account'] !== null && trim($param['account']) != '') $query[] = ['account', '=', trim($param['account'])];
        if ($param['keyWords'] !== null && trim($param['keyWords']) != '') $query[] = ['name', 'like', '%' . trim($param['keyWords']) . '%'];

        $paginator = Db::name('user')->where($query)
            ->order('user_id', 'desc')
            ->field('user_id,account,name,identity,disable_switch,create_time')
            ->paginate($param['limit'])->toArray();
        return success($paginator, '查询成功');
    }

    /**
     * @return Json
     * @throws DbException|BusinessException
     */
    public function current(): Json
    {
        $user = UserService::current();
        $user['permissions'] = $this->permissions($user);
        return success($user, '获得成功');
    }

    private function permissions($user): array
    {
        $data_admin = [
            ['permission' => '/user/list', 'type' => 'route',],
            ['permission' => '/user/list::new', 'type' => 'button',],
        ];
        $data_staff = [
        ];
        if ($user['identity'] == 2) return $data_admin;
        else return $data_staff;
    }
}