<?php

namespace app\validate;

use app\model\UserModel;
use think\Validate;

class UserValidate extends Validate
{
    protected $rule = [
        'account' => ['require', 'emptyAccount'],
        'password' => ['require'],
        'name' => ['requireSpecialName'],
        'identity' => ['require', 'in:1,2'],
        'disable_switch' => ['require', 'in:0,1'],
    ];
    protected $message = [
        'account.require' => '账号不能为空',
        'account.emptyAccount' => '账号不能重复',
        'password' => '密码不能重复',
        'name' => '系统保留的用户名',
    ];

    protected $scene = [
        'save' => ['account', 'password', 'name',],
        'edit' => ['account', 'name',],
        'identity' => ['identity'],
        'disable_switch' => ['disable_switch'],
    ];

    // 没有 require 开头的，如果参数为空，就不会验证
    protected function emptyAccount($value, $rule, $data = []): bool
    {
        if (empty($data['user_id'])) {
            $user = UserModel::where('account', $value)->find();
        } else {
            $user = UserModel::where('user_id', '<>', $data['user_id'])
                ->where('account', $value)->find();
        }
        // mysql binary
        // $user = UserModel::where('account', 'exp', "='$vale' COLLATE utf8mb4_bin")->find();
        // $user = UserModel::where('account', 'exp', "='$vale' COLLATE utf8mb4_general_ci")->find();
        return empty($user);
    }

    // require 开头的，即使参数为空也会验证
    protected function requireSpecialName($vale, $rule, $data = []): bool
    {
        return !in_array($vale, ['GuoDapeng', 'GDP']);
    }
}